function Create(self)
	function Offset(Origin, ang, dist)
		local Tmp = Vector(0, 0)
		Tmp:SetMagnitude(dist)
		Tmp:RadRotate(ang)
		Tmp = Tmp + Origin
		return Tmp
	end
	
    self.WatchTimer = Timer()
	self.Ctrl = self:GetController()
	self.Grav = SceneMan.GlobalAcc	-- GlobalAccScalar = 1
	self.Wpt = nil
	self.maxThrust = 0.45
	self.maxSpeedX = 14
	self.thrust = 0
	self.factor = FrameMan.PPM * TimerMan.DeltaTimeSecs	-- vel is expressed in PixelsPerMeter/s -> PixelsPerMeter*DeltaTime -> 20*0.0167=1/3
	
	--self.DebugDots = {}	-- debug
	self.StopFlying = 0
end

function Update(self)
  local altitude = self:GetAltitude(1000 , 10)
  if altitude < 30 and self.StopFlying < -3 then
    self.StopFlying = 1.5
  end
  self.StopFlying = self.StopFlying - 0.01
  --Stabilizer
  self.Vel.X = self.Vel.X * 0.99
  if self.Vel.Y < 0 then
    self.Vel.Y = self.Vel.Y * 0.99
  end
  self.AngularVel = self.AngularVel * 0.98
  self.RotAngle = self.RotAngle * 0.98
	local ang = nil
	if self.HFlipped then	-- left
		ang = self.RotAngle + 1.92
	else					-- right
		ang = self.RotAngle + 1.22
	end
	
	if self:IsPlayerControlled() then	
		if self.Ctrl:IsState(Controller.BODY_JUMP) then
			if self.thrust < 0.01 then
				self.thrust = self.maxThrust * 7	-- burst 17ms
			else
				self.thrust = self.maxThrust
			end
			
			self.Vel = Offset(self.Vel, ang, self.thrust)
		else
			self.thrust = 0
		end
	else
		if self.Wpt == nil then
			self.Wpt = self:GetLastAIWaypoint()
			if (self.Wpt - self.Pos).Magnitude < 5 then
				self.Wpt = nil
			elseif self.Pos.Y > self.Wpt.Y then
				self.AIMode = Actor.AIMODE_NONE
			else
				self.AIMode = Actor.AIMODE_GOTO
				self.Wpt = nil
			end
		else
			if self.WatchTimer:IsPastSimMS(100) then
				self.WatchTimer:Reset()
				if self.Pos.Y > self.Wpt.Y - 20 then	-- climb
					if self.thrust < 0.01 then
						self.thrust = self.maxThrust * 4	-- burst 100ms
					else
						self.thrust = self.maxThrust
					end
					
					if self.Vel.X > 3 then
						self.Ctrl:SetState(Controller.MOVE_LEFT, true)
						self.Ctrl:SetState(Controller.MOVE_RIGHT, false)
					elseif self.Vel.X < -3 then
						self.Ctrl:SetState(Controller.MOVE_LEFT, false)
						self.Ctrl:SetState(Controller.MOVE_RIGHT, true)
					else
						self.Ctrl:SetState(Controller.MOVE_LEFT, false)
						self.Ctrl:SetState(Controller.MOVE_RIGHT, false)
					end
				else
					local predict = false
					local range = (self.Wpt - self.Pos).Magnitude
					if self.Pos.X > self.Wpt.X + 10 then				-- move left
						if self.Vel.X < -self.maxSpeedX then -- too fast
							self.maxSpeedX = 7
							self.thrust = self.maxThrust
							self.Ctrl:SetState(Controller.MOVE_LEFT, false)
							self.Ctrl:SetState(Controller.MOVE_RIGHT, true)
						else
							self.maxSpeedX = math.max(range/25, 7)
							self.Ctrl:SetState(Controller.MOVE_LEFT, true)
							self.Ctrl:SetState(Controller.MOVE_RIGHT, false)
							
							if self.HFlipped and self.Vel.AbsRadAngle > 1.6 then -- wait until we face left and move
								predict = true
							end
						end
					elseif self.Pos.X < self.Wpt.X - 10 then			-- move right
						if self.Vel.X > self.maxSpeedX then -- too fast
							self.maxSpeedX = 7
							self.thrust = self.maxThrust
							self.Ctrl:SetState(Controller.MOVE_LEFT, true)
							self.Ctrl:SetState(Controller.MOVE_RIGHT, false)
						else
							self.maxSpeedX = math.max(range/25, 7)
							self.Ctrl:SetState(Controller.MOVE_LEFT, false)
							self.Ctrl:SetState(Controller.MOVE_RIGHT, true)
							
							if not self.HFlipped and self.Vel.AbsRadAngle < 1.54 then	-- wait until we face and move right
								predict = true								
							end
						end
					else
						self.thrust = 0
						self.Ctrl:SetState(Controller.MOVE_LEFT, false)
						self.Ctrl:SetState(Controller.MOVE_RIGHT, false)
					end
					
					-- do some arbitrary predicion of where we are going
					if predict then
						local Vel = self.Vel
						local Pos = self.Pos
						local minDist = 10000
						local dist = nil
						local Closest = nil
						
						for i = 1, 10 do
							Vel = Vel + (Vel + Offset(self.Grav/4, ang, self.thrust))*self.factor
							Pos = Pos + Vel
							dist = (self.Wpt - Pos).Magnitude
							if dist < minDist then
								minDist = dist
								Closest = Pos
							end
							
						end
						
						-- within 8 pixels is close enough to continue on the same path
						if math.abs((self.Wpt - Closest).Magnitude) > 4 then
							local bestAng = (self.Wpt - Closest).AbsRadAngle
							if bestAng < 0 or bestAng > 3.1415 then	-- we will overshoot
								self.thrust = 0
							else
								self.thrust = self.maxThrust
							end
						end
					elseif self.thrust > 0 then
						self.thrust = 0.3
					end
				end
			else
				self.Ctrl:SetState(Controller.MOVE_LEFT, false)
				self.Ctrl:SetState(Controller.MOVE_RIGHT, false)
			end
			
			if self.Wpt ~= nil then
				if self.Vel.Y > 6 then	-- try to avoid a painfull landing
					self.thrust = self.maxThrust
				end
				
				if self.thrust > 0 and (self.Pos.Y > self.Wpt.Y - 350 - self.Vel.Y^3 * 5 or altitude < 350 + self.Vel.Y^3 * 5) and self.Vel.Y > -12 and self.StopFlying < 0 then
					self.Ctrl:SetState(Controller.BODY_JUMP, true)
          if self.Health > 0 then
            self.Vel = Offset(self.Vel, ang, self.thrust)
          end
				else
					self.Ctrl:SetState(Controller.BODY_JUMP, false)
					if self.StopFlying > 0 then
            self.Ctrl:SetState(Controller.MOVE_LEFT, true)
					end
				end
			end
		end
	end
end